package com.ruoyi.bst.task.service.impl;

import java.time.LocalDateTime;
import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.support.TransactionTemplate;

import com.ruoyi.bst.task.domain.Task;
import com.ruoyi.bst.task.domain.TaskQuery;
import com.ruoyi.bst.task.domain.TaskVO;
import com.ruoyi.bst.task.domain.dto.TaskCancelDTO;
import com.ruoyi.bst.task.domain.dto.TaskStartDTO;
import com.ruoyi.bst.task.domain.dto.TaskSubmitDTO;
import com.ruoyi.bst.task.domain.dto.TaskVerifyDTO;
import com.ruoyi.bst.task.domain.enums.TaskStatus;
import com.ruoyi.bst.task.mapper.TaskMapper;
import com.ruoyi.bst.task.service.TaskService;
import com.ruoyi.bst.taskSubmit.domain.TaskSubmit;
import com.ruoyi.bst.taskSubmit.service.TaskSubmitConverter;
import com.ruoyi.bst.taskSubmit.service.TaskSubmitService;
import com.ruoyi.bst.verify.domain.Verify;
import com.ruoyi.bst.verify.service.VerifyConverter;
import com.ruoyi.bst.verify.service.VerifyService;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.common.utils.SecurityUtils;
import com.ruoyi.common.utils.ServiceUtil;
import com.ruoyi.common.vo.LongIntegerVO;
import com.ruoyi.common.vo.StringIntegerVO;

/**
 * 任务Service业务层处理
 *
 * @author ruoyi
 * @date 2025-01-22
 */
@Service
public class TaskServiceImpl implements TaskService
{
    @Autowired
    private TaskMapper taskMapper;

    @Autowired
    private TransactionTemplate transactionTemplate;

    @Autowired
    private VerifyService verifyService;

    @Autowired
    private VerifyConverter verifyConverter;

    @Autowired
    private TaskSubmitConverter taskSubmitConverter;

    @Autowired
    private TaskSubmitService taskSubmitService;

    /**
     * 查询任务
     *
     * @param id 任务主键
     * @return 任务
     */
    @Override
    public TaskVO selectTaskById(Long id)
    {
        if (id == null) {
            return null;
        }
        return taskMapper.selectTaskById(id);
    }

    /**
     * 查询任务列表
     *
     * @param task 任务
     * @return 任务
     */
    @Override
    public List<TaskVO> selectTaskList(TaskQuery task)
    {
        return taskMapper.selectTaskList(task);
    }

    /**
     * 新增任务
     *
     * @param task 任务
     * @return 结果
     */
    @Override
    public int insertTask(Task task)
    {
        task.setCreateTime(DateUtils.getNowDate());
        return taskMapper.insertTask(task);
    }

    /**
     * 修改任务
     *
     * @param task 任务
     * @return 结果
     */
    @Override
    public int updateTask(Task task)
    {
        return taskMapper.updateTask(task);
    }

    /**
     * 批量删除任务
     *
     * @param ids 需要删除的任务主键
     * @return 结果
     */
    @Override
    public int logicDel(List<Long> ids)
    {
        return taskMapper.logicDel(ids);
    }

    @Override
    public List<StringIntegerVO> selectCountGroupByStatus(TaskQuery query) {
        return taskMapper.selectCountGroupByStatus(query);
    }

    @Override
    public int selectCount(TaskQuery query) {
        return taskMapper.selectCount(query);
    }

    @Override
    public List<LongIntegerVO> selectCountGroupByProject(TaskQuery query) {
        return taskMapper.selectCountGroupByProject(query);
    }

    @Override
    public int submitTask(TaskSubmitDTO dto) {
        // 校验
        ServiceUtil.assertion(dto == null, "参数不允许为空");
        ServiceUtil.assertion(dto.getId() == null, "任务ID不允许为空");

        TaskVO old = this.selectTaskById(dto.getId());
        ServiceUtil.assertion(old == null, "ID为%s的任务不存在", dto.getId());
        ServiceUtil.assertion(!TaskStatus.canSubmit().contains(old.getStatus()), "ID为%s的任务当前状态不允许提交", dto.getId());

        // 更新任务状态
        Integer result = transactionTemplate.execute(status -> {
            // 更新状态
            Task data = new Task();
            data.setStatus(TaskStatus.WAIT_CONFIRM.getStatus());
            TaskQuery query = new TaskQuery();
            query.setId(dto.getId());
            query.setStatusList(TaskStatus.canSubmit());
            int update = this.updateByQuery(data, query);
            ServiceUtil.assertion(update != 1, "更新ID为%s的任务状态失败", dto.getId());

            // 新增提交记录
            TaskSubmit submit = taskSubmitConverter.toPo(dto);
            int insertSubmit = taskSubmitService.insertTaskSubmit(submit);
            ServiceUtil.assertion(insertSubmit != 1, "新增提交记录失败");

            return update;
        });

        return result == null ? 0 : result;
    }

    @Override
    public int startTask(TaskStartDTO dto) {
        // 校验
        ServiceUtil.assertion(dto == null, "参数不允许为空");
        ServiceUtil.assertion(dto.getId() == null, "任务ID不允许为空");

        TaskVO old = this.selectTaskById(dto.getId());
        ServiceUtil.assertion(old == null, "ID为%s的任务不存在", dto.getId());
        ServiceUtil.assertion(!TaskStatus.canStart().contains(old.getStatus()), "ID为%s的任务当前状态不允许开始", dto.getId());

        Integer result = transactionTemplate.execute(status -> {
            Task data = new Task();
            data.setStatus(TaskStatus.PROCESSING.getStatus());
            data.setExpectFinishTime(dto.getExpectFinishTime());
            TaskQuery query = new TaskQuery();
            query.setId(dto.getId());
            query.setStatusList(TaskStatus.canStart());
            int update = this.updateByQuery(data, query);
            ServiceUtil.assertion(update != 1, "更新ID为%s的任务状态失败", dto.getId());

            return update;
        });

        return result == null ? 0 : result;
    }

    private int updateByQuery(Task data, TaskQuery query) {
        return taskMapper.updateByQuery(data, query);
    }

    @Override
    public int verifyTask(TaskVerifyDTO dto) {
        // 校验
        ServiceUtil.assertion(dto == null, "参数不允许为空");
        ServiceUtil.assertion(dto.getId() == null, "任务ID不允许为空");

        TaskVO old = this.selectTaskById(dto.getId());
        ServiceUtil.assertion(old == null, "ID为%s的任务不存在", dto.getId());
        ServiceUtil.assertion(!TaskStatus.canVerify().contains(old.getStatus()), "ID为%s的任务当前状态不允许审核", dto.getId());

        Integer result = transactionTemplate.execute(status -> {
            // 新增审核记录
            Verify verify = verifyConverter.toPo(dto);
            int insertVerify = verifyService.insertVerify(verify);
            ServiceUtil.assertion(insertVerify != 1, "新增审核记录失败");

            // 更新任务状态
            Task data = new Task();
            if (dto.getPass() != null && dto.getPass()) {
                data.setStatus(TaskStatus.PASS.getStatus());
                data.setPassTime(LocalDateTime.now());
            } else {
                data.setStatus(TaskStatus.REJECT.getStatus());
            }
            data.setVerifyId(verify.getId());
            TaskQuery query = new TaskQuery();
            query.setId(dto.getId());
            query.setStatusList(TaskStatus.canVerify());
            int update = this.updateByQuery(data, query);
            ServiceUtil.assertion(update != 1, "更新ID为%s的任务状态失败", dto.getId());

            return update;
        });

        return result == null ? 0 : result;
    }

    @Override
    public int cancelTask(TaskCancelDTO dto) {  

        // 校验
        ServiceUtil.assertion(dto == null, "参数不允许为空");
        ServiceUtil.assertion(dto.getId() == null, "任务ID不允许为空");

        TaskVO old = this.selectTaskById(dto.getId());
        ServiceUtil.assertion(old == null, "ID为%s的任务不存在", dto.getId());
        ServiceUtil.assertion(!TaskStatus.canCancel().contains(old.getStatus()), "ID为%s的任务当前状态不允许取消", dto.getId());

        // 更新任务状态
        Integer result = transactionTemplate.execute(status -> {
            Task data = new Task();
            data.setStatus(TaskStatus.CANCEL.getStatus());
            data.setCancelTime(LocalDateTime.now());
            data.setCancelRemark(dto.getCancelRemark());
            data.setCancelUserId(SecurityUtils.getUserId());
            TaskQuery query = new TaskQuery();
            query.setId(dto.getId());
            query.setStatusList(TaskStatus.canCancel());
            int update = this.updateByQuery(data, query);
            ServiceUtil.assertion(update != 1, "更新ID为%s的任务状态失败", dto.getId());

            return update;
        });

        return result == null ? 0 : result;
    }
}
